@@ -83,6 +83,14 @@ GList *             path_list_dup                (GList *path_list);
 gboolean            is_mime_type                 (const char* type, 
 						  const char* pattern);
 
+gboolean            strchrs                      (const char *str,
+						  const char *chars);
+
+char*               escape_str_common            (const char *str, 
+						  const char *meta_chars,
+						  const char  prefix,
+						  const char  postfix);
+
 char*               escape_str                   (const char  *str, 
 						  const char  *meta_chars);
 
@@ -115,5 +123,24 @@ char *              get_temp_work_dir            (void);
 
 char *              get_temp_work_dir_name       (void);
 
+char *              str_substitute               (const char *str,
+						  const char *from_str,
+						  const char *to_str);
+
+/* misc functions used to parse a command output lines. */
+
+gboolean            file_list__match_pattern     (const char *line, 
+						  const char *pattern);
+
+int                 file_list__get_index_from_pattern (const char *line, 
+						       const char *pattern);
+
+char*               file_list__get_next_field    (const char *line,
+						  int         start_from,
+						  int         field_n);
+
+char*               file_list__get_prev_field    (const char *line,
+						  int         start_from,
+						  int         field_n);
 
 #endif /* FILE_UTILS_H */
