@@ -24,6 +24,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <stdio.h>
+#include <sys/param.h>
 #include <glib.h>
 #include <gnome.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
@@ -32,6 +33,7 @@
 #include "file-utils.h"
 #include "fr-archive.h"
 #include "fr-command.h"
+#include "fr-command-ar.h"
 #include "fr-command-arj.h"
 #include "fr-command-cfile.h"
 #include "fr-command-iso.h"
@@ -42,19 +44,19 @@
 #include "fr-command-unstuff.h"
 #include "fr-command-zip.h"
 #include "fr-command-zoo.h"
+#include "fr-command-7z.h"
 #include "fr-error.h"
 #include "fr-marshal.h"
 #include "fr-process.h"
 #include "utf8-fnmatch.h"
 
 
-#define MAX_CHUNK_LEN 16000 /* FIXME : what is the max length of a command 
-			     * line ? */
+#define MAX_CHUNK_LEN (NCARGS - PATH_MAX) /* Max length of the command line */
 #define UNKNOWN_TYPE "application/octet-stream"
 #define SAME_FS (FALSE)
-#define NO_BACKUP_FILES (TRUE)
-#define NO_DOT_FILES (TRUE)
-#define IGNORE_CASE (TRUE)
+#define NO_BACKUP_FILES (FALSE)
+#define NO_DOT_FILES (FALSE)
+#define IGNORE_CASE (FALSE)
 
 
 enum {
@@ -276,6 +278,13 @@ create_command_from_mime_type (FRArchive  *archive,
 	} else if (is_mime_type (mime_type, "application/x-cd-image")) {
 		archive->command = fr_command_iso_new (archive->process,
 						       filename);
+	} else if (is_mime_type (mime_type, "application/x-deb") ||
+		   is_mime_type (mime_type, "application/x-ar")) {
+		archive->command = fr_command_ar_new (archive->process, 
+						      filename);
+	} else if (is_mime_type (mime_type, "application/x-7zip")) {
+		archive->command = fr_command_7z_new (archive->process,
+						      filename);
 	} else 
 		return FALSE;
 
@@ -410,6 +419,12 @@ create_command_from_filename (FRArchive  *archive,
 	} else if (file_extension_is (filename, ".arj")) {
 		archive->command = fr_command_arj_new (archive->process, 
 						       filename);
+	} else if (file_extension_is (filename, ".ar")) {
+		archive->command = fr_command_ar_new (archive->process, 
+						      filename);
+	} else if (file_extension_is (filename, ".7z")) {
+		archive->command = fr_command_7z_new (archive->process,
+						      filename);
 	} else if (loading) {
 		if (file_extension_is (filename, ".gz")
 		    || file_extension_is (filename, ".z")
@@ -435,6 +450,10 @@ create_command_from_filename (FRArchive  *archive,
 		} else if (file_extension_is (filename, ".iso")) {
 			archive->command = fr_command_iso_new (archive->process,
 							       filename);
+		} else if (file_extension_is (filename, ".deb")) {
+			archive->command = fr_command_ar_new (archive->process, 
+							      filename);
+
 		} else
 			return FALSE;
 	} else
@@ -778,14 +797,33 @@ static void _archive_remove (FRArchive *archive, GList *file_list);
 
 
 static GList *
-escape_file_list (GList *file_list) 
+escape_file_list (FRCommand *command,
+		  GList     *file_list) 
 {
 	GList *e_file_list = NULL;
 	GList *scan;
 
 	for (scan = file_list; scan; scan = scan->next) {
 		char *filename = scan->data;
-		e_file_list = g_list_prepend (e_file_list, shell_escape (filename));
+		char *escape = fr_command_escape (command, filename);
+		e_file_list = g_list_prepend (e_file_list, escape);
+	}
+
+	return e_file_list;
+}
+
+
+static GList *
+shell_escape_file_list (FRCommand *command,
+			GList     *file_list) 
+{
+	GList *e_file_list = NULL;
+	GList *scan;
+
+	for (scan = file_list; scan; scan = scan->next) {
+		char *filename = scan->data;
+		char *escape = shell_escape (filename);
+		e_file_list = g_list_prepend (e_file_list, escape);
 	}
 
 	return e_file_list;
@@ -932,7 +970,7 @@ fr_archive_add (FRArchive     *archive,
 
 	/* add now. */
 
-	e_file_list = escape_file_list (new_file_list);
+	e_file_list = shell_escape_file_list (archive->command, new_file_list);
 	fr_command_set_n_files (archive->command, g_list_length (e_file_list));
 
 	for (scan = e_file_list; scan != NULL; ) {
@@ -995,25 +1033,17 @@ file_list_remove_from_pattern (GList      **list,
 	for (scan = *list; scan;) {
 		char *path = scan->data;
 		char *utf8_name;
-
+		
 		utf8_name = g_filename_to_utf8 (file_name_from_path (path), 
 						-1, NULL, NULL, NULL);
 
-		if (strcmp ("lt-gthumb", utf8_name) == 0)
-			g_print ("%s <--> %s\n", pattern, utf8_name);
-
-		if (match_patterns (patterns, utf8_name, FNM_CASEFOLD)) {
+		if (match_patterns (patterns, utf8_name, 0)) {
 			*list = g_list_remove_link (*list, scan);
 			g_free (scan->data);
 			g_list_free (scan);
 			scan = *list;
-			if (strcmp ("lt-gthumb", utf8_name) == 0)
-				g_print ("Y\n");
-		} else {
+		} else 
 			scan = scan->next;
-			if (strcmp ("lt-gthumb", utf8_name) == 0)
-				g_print ("N\n");
-		}
 
 		g_free (utf8_name);
 	}
@@ -1073,11 +1103,6 @@ fr_archive_add_with_wildcard (FRArchive     *archive,
 			      gboolean       update,
 			      gboolean       recursive,
 			      gboolean       follow_links,
-			      /*gboolean       same_fs,
-			      gboolean       no_backup_files,
-			      gboolean       no_dot_files,
-			      gboolean       ignore_case,
-			      */
 			      const char    *password,
 			      FRCompression  compression,
 			      DoneFunc       done_func,
@@ -1218,7 +1243,7 @@ _archive_remove (FRArchive *archive,
 		file_list_created = TRUE;
 	}
 
-	e_file_list = escape_file_list (file_list);
+	e_file_list = escape_file_list (archive->command, file_list);
 	fr_command_set_n_files (archive->command, g_list_length (e_file_list));
 
 	for (scan = e_file_list; scan != NULL; ) {
@@ -1283,21 +1308,23 @@ move_files_to_dir (FRArchive  *archive,
 	fr_process_add_arg (archive->process, "-f");
 	for (scan = file_list; scan; scan = scan->next) {
 		char  path[4096]; /* FIXME : 4096 ? */
-		char *filename = scan->data;
+		char *e_filename = shell_escape (scan->data);
 		
-		if (filename[0] == '/')
-			sprintf (path, "%s%s", source_dir, filename);
+		if (e_filename[0] == '/')
+			sprintf (path, "%s%s", source_dir, e_filename);
 		else
-			sprintf (path, "%s/%s", source_dir, filename);
+			sprintf (path, "%s/%s", source_dir, e_filename);
 		
 		fr_process_add_arg (archive->process, path);
+
+		g_free (e_filename);
 	}
 	fr_process_add_arg (archive->process, dest_dir);
 	fr_process_end_command (archive->process);
 }
 
 
-/* Note: all paths escaped, temp_dir and dest_dir unescaped. */
+/* Note: all paths unescaped, temp_dir and dest_dir unescaped. */
 static void
 move_files_in_chunks (FRArchive  *archive,
 		      GList      *file_list,
@@ -1336,7 +1363,7 @@ move_files_in_chunks (FRArchive  *archive,
 				   e_dest_dir);
 		prev->next = scan;
 	}	
-	
+
 	g_free (e_temp_dir);
 	g_free (e_dest_dir);
 }
@@ -1397,12 +1424,12 @@ extract_in_chunks (FRCommand  *command,
 
 
 static char*
-compute_base_path (const char *e_base_dir,
+compute_base_path (const char *base_dir,
 		   const char *path,
 		   gboolean    junk_paths,
 		   gboolean    can_junk_paths)
 {
-	int         e_base_dir_len = strlen (e_base_dir);
+	int         base_dir_len = strlen (base_dir);
 	int         path_len = strlen (path);
 	const char *base_path;
 	char       *name_end;
@@ -1414,14 +1441,15 @@ compute_base_path (const char *e_base_dir,
 		else
 			new_path = g_strdup (path);
 #ifdef DEBUG
-		g_print ("%s, %s --> %s\n", e_base_dir, path, new_path);
+		g_print ("%s, %s --> %s\n", base_dir, path, new_path);
 #endif
 		return new_path;
 	}
 
-	if (path_len <= e_base_dir_len)
+	if (path_len <= base_dir_len)
 		return NULL;
-	base_path = path + e_base_dir_len;
+
+	base_path = path + base_dir_len;
 	if (path[0] != '/')
 		base_path -= 1;
 	name_end = strchr (base_path, '/');
@@ -1434,7 +1462,7 @@ compute_base_path (const char *e_base_dir,
 	}
 
 #ifdef DEBUG
-	g_print ("%s, %s --> %s\n", e_base_dir, path, new_path);
+	g_print ("%s, %s --> %s\n", base_dir, path, new_path);
 #endif
 
 	return new_path;
@@ -1443,22 +1471,21 @@ compute_base_path (const char *e_base_dir,
 
 static GList*
 compute_list_base_path (const char *base_dir,
-			GList      *e_filtered,
+			GList      *filtered,
 			gboolean    junk_paths,
 			gboolean    can_junk_paths)
 {
-	char  *e_base_dir = shell_escape (base_dir);
 	GList *scan;
 	GList *list = NULL, *list_unique = NULL;
 	GList *last_inserted;
 
-	if (e_filtered == NULL)
+	if (filtered == NULL)
 		return NULL;
 
-	for (scan = e_filtered; scan; scan = scan->next) {
+	for (scan = filtered; scan; scan = scan->next) {
 		const char *path = scan->data;
 		char       *new_path;
-		new_path = compute_base_path (e_base_dir, path, junk_paths, can_junk_paths);
+		new_path = compute_base_path (base_dir, path, junk_paths, can_junk_paths);
 		if (new_path != NULL)
 			list = g_list_prepend (list, new_path);
 	}
@@ -1483,7 +1510,6 @@ compute_list_base_path (const char *base_dir,
 	}
 
 	g_list_free (list);
-	g_free (e_base_dir);
 
 	return list_unique;
 }
@@ -1539,7 +1565,7 @@ fr_archive_extract (FRArchive  *archive,
 	    && ! (junk_paths && ! archive->command->propExtractCanJunkPaths)) {
 		GList *e_file_list;
 
-		e_file_list = escape_file_list (file_list);
+		e_file_list = escape_file_list (archive->command, file_list);
 		extract_in_chunks (archive->command,
 				   e_file_list,
 				   dest_dir,
@@ -1631,7 +1657,7 @@ fr_archive_extract (FRArchive  *archive,
 		return;
 	} 
 
-	e_filtered = escape_file_list (filtered);	
+	e_filtered = escape_file_list (archive->command, filtered);	
 
 	if (move_to_dest_dir) {
 		char *temp_dir;
@@ -1647,13 +1673,13 @@ fr_archive_extract (FRArchive  *archive,
 				   password);
 
 		if (use_base_dir) {
-			GList *tmp = compute_list_base_path (base_dir, e_filtered, junk_paths, archive->command->propExtractCanJunkPaths);
-			path_list_free (e_filtered);
-			e_filtered = tmp;
+			GList *tmp_list = compute_list_base_path (base_dir, filtered, junk_paths, archive->command->propExtractCanJunkPaths);
+			g_list_free (filtered);
+			filtered = tmp_list;
 		}
 
 		move_files_in_chunks (archive, 
-				      e_filtered, 
+				      filtered, 
 				      temp_dir, 
 				      dest_dir);
 
@@ -1708,10 +1734,13 @@ G_CONST_RETURN char *
 fr_archive_utils__get_file_name_ext (const char *filename)
 {
 	static char * ext[] = {
+		".7z",
+		".ar",
 		".arj",
 		".bin",
 		".bz", 
 		".bz2", 
+		".deb",
 		".ear",
 		".gz", 
 		".iso",
