@@ -114,10 +114,6 @@ VisitDirHandle * fr_archive_add_with_wildcard  (FRArchive     *archive,
 						gboolean       update,
 						gboolean       recursive,
 						gboolean       follow_links,
-						gboolean       same_fs,
-						gboolean       no_backup_files,
-						gboolean       no_dot_files,
-						gboolean       ignore_case,
 						const char    *password,
 						FRCompression  compression,
 						DoneFunc       done_func,
@@ -140,6 +136,7 @@ void             fr_archive_remove             (FRArchive     *archive,
 void             fr_archive_extract            (FRArchive     *archive,
 						GList         *file_list,
 						const char    *dest_dir,
+						const char    *base_dir,
 						gboolean       skip_older,
 						gboolean       overwrite,
 						gboolean       junk_path,
