@@ -130,6 +130,14 @@ base_fr_command_recompress (FRCommand     *comm,
 }
 
 
+char *
+base_fr_command_escape (FRCommand     *comm,
+			const char    *str)
+{
+	return shell_escape (str);
+}
+
+
 static void
 base_fr_command_handle_error (FRCommand *comm, 
 			      FRProcError *error)
@@ -193,6 +201,7 @@ fr_command_class_init (FRCommandClass *class)
 	class->uncompress     = base_fr_command_uncompress;
 	class->recompress     = base_fr_command_recompress;
 
+	class->escape         = base_fr_command_escape;
 	class->handle_error   = base_fr_command_handle_error;
 
 	class->start          = NULL;
@@ -218,6 +227,7 @@ fr_command_init (FRCommand *comm)
 	comm->propExtractCanJunkPaths = FALSE;
 	comm->propPassword = FALSE;
 	comm->propTest = FALSE;
+	comm->propCanExtractAll = TRUE;
 }
 
 
@@ -468,6 +478,14 @@ fr_command_recompress (FRCommand     *comm,
 }
 
 
+char *
+fr_command_escape (FRCommand     *comm,
+		   const char    *str)
+{
+	return FR_COMMAND_GET_CLASS (G_OBJECT (comm))->escape (comm, str);
+}
+
+
 /* fraction == -1 means : I don't known how much time the current operation will take, 
  *                        the dialog will display this info pulsing the progress bar. 
  * fraction in [0.0, 1.0] means the amount of work, in percentage, accomplished. */
